from urllib.parse import urlencode, parse_qsl
from datetime import datetime, timedelta
from pytz import UTC
from dateutil.parser import parse as parse_datetime
from sqlalchemy.exc import InternalError, IntegrityError, DataError
from sqlalchemy import and_, or_, inspect, alias, text, func, select, case, cast
from sqlalchemy.orm import aliased
from api_dnl import model
import falcon
from falcon_rest import conf
from falcon_rest.logger import log
from falcon_rest import schemes
from falcon_rest.responses import responses
from falcon_rest.responses import errors
from falcon_rest.resources.resources import swagger, ATTRIBUTE_ERROR_RE
from falcon_rest.helpers import check_permission

from api_dnl.resources import check_context, DnlResource,DnlList, ValidationError, DnlCreate, CustomGetAction, CustomPostAction, \
    CustomDeleteAction
from api_dnl.schemes.shaken import *
from api_dnl.scheme import SendCdrDirectScheme
from api_dnl.view import DEFAULT_SECURITY, OperationErrorResponse
import json
from api_dnl.views.report import Report


# region +++C4ShakenConf+++
class C4ShakenConfCreate(DnlCreate):
    scheme_class = C4ShakenConfScheme
    model_class = model.C4ShakenConf
    entity = 'C4ShakenConf'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()

    def before_create(self, obj, **kwargs):
        user = self.get_user(self.req)
        # obj.created_by=user.name
        # obj.created_on=datetime.now(UTC)
        return obj


class C4ShakenConfResource(DnlResource):
    model_class = model.C4ShakenConf
    scheme_class = C4ShakenConfScheme
    scheme_class_get = C4ShakenConfSchemeGet
    scheme_class_modify = C4ShakenConfSchemeModify
    entity = 'C4ShakenConf'
    id_field = 'id'
    security = (DEFAULT_SECURITY)
    path_parameters = ()
    restrict = ()


class C4ShakenConfList(DnlList):
    scheme_class = C4ShakenConfSchemeGet
    model_class = model.C4ShakenConf
    entity_plural = 'C4ShakenConfs'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()

    def modify_query_from_filtering_for_list(self, filtering, **kwargs):
        filt, ret = super().modify_query_from_filtering_for_list(filtering, **kwargs)
        user = self.get_user(self.req)
        if not user.is_admin:
            cls = self.model_class

        return filt, ret


# endregion ---C4ShakenConf---

#region +++ShakenAniGroup+++
class ShakenAniGroupCreate(DnlCreate):
    scheme_class = ShakenAniGroupScheme
    model_class = model.ShakenAniGroup
    entity = 'ShakenAniGroup'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()
    def before_create(self, obj, **kwargs):
        user = self.get_user(self.req)
        #obj.created_by=user.name
        #obj.created_on=datetime.now(UTC)
        return obj
class ShakenAniGroupResource(DnlResource):
    model_class = model.ShakenAniGroup
    scheme_class = ShakenAniGroupScheme
    scheme_class_get = ShakenAniGroupSchemeGet
    scheme_class_modify = ShakenAniGroupSchemeModify
    entity = 'ShakenAniGroup'
    id_field = 'id'
    security = (DEFAULT_SECURITY)
    path_parameters = ()
    restrict = ()
class ShakenAniGroupList(DnlList):
    scheme_class = ShakenAniGroupSchemeGet
    model_class = model.ShakenAniGroup
    entity_plural = 'ShakenAniGroups'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()
    def modify_query_from_filtering_for_list(self, filtering, **kwargs):
        filt, ret = super().modify_query_from_filtering_for_list(filtering, **kwargs)
        user = self.get_user(self.req)
        if not user.is_admin:
            cls = self.model_class
            #ret = ret.filter(cls.pool_id != 0)#TODO:filter for user
        return filt, ret
#endregion ---ShakenAniGroup---

#region +++ShakenAniGroupList+++
class ShakenAniGroupListCreate(DnlCreate):
    scheme_class = ShakenAniGroupListScheme
    model_class = model.ShakenAniGroupList
    entity = 'ShakenAniGroupList'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()
    def before_create(self, obj, **kwargs):
        user = self.get_user(self.req)
        #obj.created_by=user.name
        #obj.created_on=datetime.now(UTC)
        return obj
class ShakenAniGroupListResource(DnlResource):
    model_class = model.ShakenAniGroupList
    scheme_class = ShakenAniGroupListScheme
    scheme_class_get = ShakenAniGroupListSchemeGet
    scheme_class_modify = ShakenAniGroupListSchemeModify
    entity = 'ShakenAniGroupList'
    id_field = 'id'
    security = (DEFAULT_SECURITY)
    path_parameters = ()
    restrict = ()
class ShakenAniGroupListList(DnlList):
    scheme_class = ShakenAniGroupListSchemeGet
    model_class = model.ShakenAniGroupList
    entity_plural = 'ShakenAniGroupLists'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()
    def modify_query_from_filtering_for_list(self, filtering, **kwargs):
        filt, ret = super().modify_query_from_filtering_for_list(filtering, **kwargs)
        user = self.get_user(self.req)
        if not user.is_admin:
            cls = self.model_class
            #ret = ret.filter(cls.pool_id != 0)#TODO:filter for user
        return filt, ret
#endregion ---ShakenAniGroupList---

#region +++ShakenAniGroupListRel+++
class ShakenAniGroupListRelCreate(DnlCreate):
    scheme_class = ShakenAniGroupListRelScheme
    model_class = model.ShakenAniGroupListRel
    entity = 'ShakenAniGroupListRel'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()
    def before_create(self, obj, **kwargs):
        user = self.get_user(self.req)
        #obj.created_by=user.name
        #obj.created_on=datetime.now(UTC)
        return obj
class ShakenAniGroupListRelResource(DnlResource):
    model_class = model.ShakenAniGroupListRel
    scheme_class = ShakenAniGroupListRelScheme
    scheme_class_get = ShakenAniGroupListRelSchemeGet
    scheme_class_modify = ShakenAniGroupListRelSchemeModify
    entity = 'ShakenAniGroupListRel'
    id_field = 'ani_group_list_id'
    security = (DEFAULT_SECURITY)
    path_parameters = ()
    restrict = ()
class ShakenAniGroupListRelList(DnlList):
    scheme_class = ShakenAniGroupListRelSchemeGet
    model_class = model.ShakenAniGroupListRel
    entity_plural = 'ShakenAniGroupListRels'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()
    def modify_query_from_filtering_for_list(self, filtering, **kwargs):
        filt, ret = super().modify_query_from_filtering_for_list(filtering, **kwargs)
        user = self.get_user(self.req)
        if not user.is_admin:
            cls = self.model_class
            #ret = ret.filter(cls.pool_id != 0)#TODO:filter for user
        return filt, ret
#endregion ---ShakenAniGroupListRel---

#region +++ShakenAniGroupRel+++
class ShakenAniGroupRelCreate(DnlCreate):
    scheme_class = ShakenAniGroupRelScheme
    model_class = model.ShakenAniGroupRel
    entity = 'ShakenAniGroupRel'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()
    def before_create(self, obj, **kwargs):
        user = self.get_user(self.req)
        #obj.created_by=user.name
        #obj.created_on=datetime.now(UTC)
        return obj
class ShakenAniGroupRelResource(DnlResource):
    model_class = model.ShakenAniGroupRel
    scheme_class = ShakenAniGroupRelScheme
    scheme_class_get = ShakenAniGroupRelSchemeGet
    scheme_class_modify = ShakenAniGroupRelSchemeModify
    entity = 'ShakenAniGroupRel'
    id_field = 'group_id'
    security = (DEFAULT_SECURITY)
    path_parameters = ()
    restrict = ()
class ShakenAniGroupRelList(DnlList):
    scheme_class = ShakenAniGroupRelSchemeGet
    model_class = model.ShakenAniGroupRel
    entity_plural = 'ShakenAniGroupRels'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()
    def modify_query_from_filtering_for_list(self, filtering, **kwargs):
        filt, ret = super().modify_query_from_filtering_for_list(filtering, **kwargs)
        user = self.get_user(self.req)
        if not user.is_admin:
            cls = self.model_class
            #ret = ret.filter(cls.pool_id != 0)#TODO:filter for user
        return filt, ret
#endregion ---ShakenAniGroupRel---

### end ###
